var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script><script src=\"../../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /><meta http-equiv=\"Content-Style-Type\" content=\"text/css\" /><meta name=\"generator\" content=\"pandoc\" /><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Handling Deprecated API Calls</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script>$(document).ready(function () { prettyPrint(); } );</script><script></script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Handling Deprecated API Calls</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"handling-deprecated-api-calls\"></a></div>\n\
<p>When you use a deprecated API call, a warning such as this one will be generated by your compiler:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-sh\">warning C4996: &#39;<a href=\"javascript:void(0)\" data-symbol=\"MPxTransform::validateAndSetValue\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html#ace2cd11d9d4eade0bcb57a2e341faedb&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform::validateAndSetValue</a>&#39;: Override validateAndSetValue(const <a href=\"javascript:void(0)\" data-symbol=\"MPlug\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_plug.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPlug</a>&amp;, const <a href=\"javascript:void(0)\" data-symbol=\"MDataHandle\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_data_handle.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDataHandle</a>&amp;) instead.  If needed, useMDGContext::current() to get the context.\n\
</pre></div><p>If you enabled <span class=\'code\'>treat warning as error</span> on your compiler and experienced a related build error during your migration, you can disable the deprecation warnings.</p>\n\
<p>To silence the deprecation warnings, you can define the <span class=\'code\'>_OPENMAYA_DEPRECATION_DISABLE_WARNING</span> preprocessor macro, or use one of the following compiler switches:</p>\n\
<table class=\'ruled\'>\n\
<tr class=\"odd\">\n\
<td align=\"left\">MSVC</td>\n\
<td align=\"left\">/Wd&quot;4996&quot;</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td align=\"left\">ICC</td>\n\
<td align=\"left\">-diag-disable 1786</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td align=\"left\">Clang</td>\n\
<td align=\"left\">-Wno-error=deprecated-declarations</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td align=\"left\">GCC</td>\n\
<td align=\"left\">-Wno-error=deprecated-declarations</td>\n\
</tr>\n\
</table>\n\
<p>To treat deprecation as an error, your can define the <span class=\'code\'>_OPENMAYA_DEPRECATION_ERROR_ALL</span> preprocessor macro, or use one of the following compiler switches:</p>\n\
<table class=\'ruled\'>\n\
<tr class=\"odd\">\n\
<td align=\"left\">MSVC</td>\n\
<td align=\"left\">/We&quot;4996&quot; (/D_CRT_NONSTDC_NO_DEPRECATE)</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td align=\"left\">ICC</td>\n\
<td align=\"left\">-diag-error 1786</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td align=\"left\">Clang</td>\n\
<td align=\"left\">-Werror=deprecated-declarations</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td align=\"left\">GCC</td>\n\
<td align=\"left\">Werror=deprecated-declarations</td>\n\
</tr>\n\
</table>\n\
<div class=\'section\'><a id=\"backwards-compatibility-for-deprecated-methods\"></a><h2 id=\"backwards-compatibility-for-deprecated-methods\">Backwards compatibility for deprecated methods</h2></div>\n\
<p>Deprecated methods can be removed from the API as soon as within three major releases. As such you should remove deprecated methods from your plug-ins as soon as possible.</p>\n\
<p>However, if the scope of the change is too great to be done all at once, or if you are using third-party software, you may have deprecated API calls remaining in your code.</p>\n\
<p>You can override deprecated API calls as long as you do not override both the deprecated and the new API in the same hierarchy or in the same class.</p>\n\
<p>The following examples show how to override a new and deprecated API. The examples use the <span class=\'code\'>getMatrix()</span> method. <span class=\'code\'>MMatrix::getMatrix(const <a href=\"javascript:void(0)\" data-symbol=\"MDGContext\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;},{&quot;path&quot;:&quot;py_ref/class_open_maya_1_1_m_d_g_context.html&quot;,&quot;title&quot;:&quot;Python 2.0 API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext</a>&amp;)</span> was deprecated in Maya 2018 and replaced with <span class=\'code\'>MMatrix::getMatrix()</span>.</p>\n\
<p>To override the new API:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">class UserTransform : public <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform</a> {\n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix() override;\n\
};\n\
</pre></div><p>To override the deprecated API:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">class UserTransform: public <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform</a> {\n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix(const <a href=\"javascript:void(0)\" data-symbol=\"MDGContext\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;},{&quot;path&quot;:&quot;py_ref/class_open_maya_1_1_m_d_g_context.html&quot;,&quot;title&quot;:&quot;Python 2.0 API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext</a>&amp;) override; // deprecated\n\
};\n\
</pre></div><div class=\'section\'><a id=\"common-mistakes-to-avoid\"></a><h2 id=\"common-mistakes-to-avoid\">Common mistakes to avoid</h2></div>\n\
<p>When using both a new and deprecated API, do not override both the new and deprecated APIs within the same class or inheritance hierarchy, and do not call the deprecated API from the new API&#39;s override or vice versa.</p>\n\
<p>The following examples illustrate these common mistakes. The examples use the <span class=\'code\'>getMatrix()</span> method. <span class=\'code\'>MMatrix::getMatrix(const <a href=\"javascript:void(0)\" data-symbol=\"MDGContext\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;},{&quot;path&quot;:&quot;py_ref/class_open_maya_1_1_m_d_g_context.html&quot;,&quot;title&quot;:&quot;Python 2.0 API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext</a>&amp;)</span> was deprecated in Maya 2018 and replaced with <span class=\'code\'>MMatrix::getMatrix()</span>.</p>\n\
<p>Do not override the new and deprecated APIs within the same class. For example DO NOT do this:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">class UserTransform: public <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform</a> { \n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix() override; // new \n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix(const <a href=\"javascript:void(0)\" data-symbol=\"MDGContext\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;},{&quot;path&quot;:&quot;py_ref/class_open_maya_1_1_m_d_g_context.html&quot;,&quot;title&quot;:&quot;Python 2.0 API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext</a>&amp;) override; // deprecated \n\
}; // Error, override both API in one class \n\
</pre></div><p>Do not override both the new and deprecated APIs in the same inheritance hierarchy.</p>\n\
<p>For example, DO NOT do this:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">class UserTransformBase: public <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform</a> { \n\
   <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix(const <a href=\"javascript:void(0)\" data-symbol=\"MDGContext\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;},{&quot;path&quot;:&quot;py_ref/class_open_maya_1_1_m_d_g_context.html&quot;,&quot;title&quot;:&quot;Python 2.0 API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext</a>&amp;) override; // deprecated \n\
}; // Ok \n\
\n\
class UserTransformDerived: public UserTransformBase {  \n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix() override; // new \n\
}; // Error, override both API in one inheritance hierarchy \n\
</pre></div><p>Do not call the deprecated API from new API’s override or call the new API from the deprecated API override:</p>\n\
<p>For example, DO NOT do this:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">class UserTransform: public <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform</a> { \n\
    <a href=\"javascript:void(0)\" data-symbol=\"MMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_matrix.html&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MMatrix</a> getMatrix() override { \n\
        return <a href=\"javascript:void(0)\" data-symbol=\"MPxTransform::getMatrix\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_px_transform.html#af6e52bdebc5f622b643dfdf60d9e15fb&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MPxTransform::getMatrix</a>(<a href=\"javascript:void(0)\" data-symbol=\"MDGContext::current\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_d_g_context.html#a11a98edcfc027fb57d3a66fcfbc580a8&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MDGContext::current</a>()); // Error, calling deprecated API from new API overridden \n\
    } \n\
};\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";